@@ -1,5 +1,7 @@
 import math
 import itertools
+
+import outcome
 from contextlib import contextmanager
 import socket as stdlib_socket
 from select import select
@@ -20,6 +22,7 @@ from ._windows_cffi import (
     INVALID_HANDLE_VALUE,
     raise_winerror,
     ErrorCodes,
+    _handle,
 )
 
 # There's a lot to be said about the overall design of a Windows event
@@ -94,19 +97,6 @@ def _check(success):
     return success
 
 
-def _handle(obj):
-    # For now, represent handles as either cffi HANDLEs or as ints.  If you
-    # try to pass in a file descriptor instead, it's not going to work
-    # out. (For that msvcrt.get_osfhandle does the trick, but I don't know if
-    # we'll actually need that for anything...) For sockets this doesn't
-    # matter, Python never allocates an fd. So let's wait until we actually
-    # encounter the problem before worrying about it.
-    if type(obj) is int:
-        return ffi.cast("HANDLE", obj)
-    else:
-        return obj
-
-
 @attr.s(frozen=True)
 class _WindowsStatistics:
     tasks_waiting_overlapped = attr.ib()
@@ -191,8 +181,7 @@ class WindowsIOManager:
         # If there are events queued from the IOCP thread, then the timeout is
         # implicitly reduced to 0 b/c the wakeup socket has pending data in
         # it.
-
-        def socket_ready(what, sock, result=_core.Value(None)):
+        def socket_ready(what, sock, result):
             task = self._socket_waiters[what].pop(sock)
             _core.reschedule(task, result)
 
@@ -200,7 +189,7 @@ class WindowsIOManager:
             try:
                 select([sock], [sock], [sock], 0)
             except OSError as exc:
-                socket_ready(what, sock, result=_core.Error(exc))
+                socket_ready(what, sock, outcome.Error(exc))
 
         def do_select():
             r_waiting = self._socket_waiters["read"]
@@ -224,9 +213,9 @@ class WindowsIOManager:
 
         for sock in r:
             if sock is not self._main_thread_waker.wakeup_sock:
-                socket_ready("read", sock)
+                socket_ready("read", sock, outcome.Value(None))
         for sock in w:
-            socket_ready("write", sock)
+            socket_ready("write", sock, outcome.Value(None))
 
         # Step 2: drain the wakeup socket.
         # This must be done before checking the IOCP queue.
@@ -381,6 +370,19 @@ class WindowsIOManager:
     async def wait_socket_writable(self, sock):
         await self._wait_socket("write", sock)
 
+    @_public
+    def notify_socket_close(self, sock):
+        if type(sock) is not stdlib_socket.socket:
+            raise TypeError("need a stdlib socket")
+
+        for mode in ["read", "write"]:
+            if sock in self._socket_waiters[mode]:
+                task = self._socket_waiters[mode].pop(sock)
+                exc = _core.ClosedResourceError(
+                    "another task closed this socket"
+                )
+                _core.reschedule(task, outcome.Error(exc))
+
     # This has cffi-isms in it and is untested... but it demonstrates the
     # logic we'll want when we start actually using overlapped I/O.
     #
