@@ -1,19 +1,26 @@
 import functools
-import inspect
 import logging
 import os
 import random
 import select
 import threading
 from collections import deque
+import collections.abc
 from contextlib import contextmanager, closing
+
+import outcome
 from contextvars import copy_context
 from math import inf
 from time import monotonic
 
+from sniffio import current_async_library_cvar
+
 import attr
-from async_generator import async_generator, yield_, asynccontextmanager
+from async_generator import (
+    async_generator, yield_, asynccontextmanager, isasyncgen
+)
 from sortedcontainers import SortedDict
+from outcome import Error, Value
 
 from . import _public
 from ._entry_queue import EntryQueue, TrioToken
@@ -23,7 +30,6 @@ from ._ki import (
     enable_ki_protection
 )
 from ._multierror import MultiError
-from ._result import Result, Error, Value
 from ._traps import (
     Abort,
     wait_task_rescheduled,
@@ -402,7 +408,7 @@ class Nursery:
             # KeyboardInterrupt), then save that, but still wait until our
             # children finish.
             def aborted(raise_cancel):
-                self._add_exc(Result.capture(raise_cancel).error)
+                self._add_exc(outcome.capture(raise_cancel).error)
                 return Abort.FAILED
 
             self._parent_waiting_in_aexit = True
@@ -504,7 +510,7 @@ class Task:
     @property
     def parent_nursery(self):
         """The nursery this task is inside (or None if this is the "init"
-        take).
+        task).
 
         Example use case: drawing a visualization of the task tree in a
         debugger.
@@ -542,7 +548,7 @@ class Task:
         # whether we succeeded or failed.
         self._abort_func = None
         if success is Abort.SUCCEEDED:
-            self._runner.reschedule(self, Result.capture(raise_cancel))
+            self._runner.reschedule(self, outcome.capture(raise_cancel))
 
     def _attempt_delivery_of_any_pending_cancel(self):
         if self._abort_func is None:
@@ -604,12 +610,15 @@ class Runner:
     init_task = attr.ib(default=None)
     init_task_result = attr.ib(default=None)
     system_nursery = attr.ib(default=None)
+    system_context = attr.ib(default=None)
     main_task = attr.ib(default=None)
     main_task_result = attr.ib(default=None)
 
     entry_queue = attr.ib(default=attr.Factory(EntryQueue))
     trio_token = attr.ib(default=None)
 
+    _NO_SEND = object()
+
     def close(self):
         self.io_manager.close()
         self.entry_queue.close()
@@ -688,9 +697,9 @@ class Runner:
     ################
 
     @_public
-    def reschedule(self, task, next_send=Value(None)):
+    def reschedule(self, task, next_send=_NO_SEND):
         """Reschedule the given task with the given
-        :class:`~trio.hazmat.Result`.
+        :class:`outcome.Outcome`.
 
         See :func:`wait_task_rescheduled` for the gory details.
 
@@ -702,10 +711,13 @@ class Runner:
         Args:
           task (trio.hazmat.Task): the task to be rescheduled. Must be blocked
             in a call to :func:`wait_task_rescheduled`.
-          next_send (trio.hazmat.Result): the value (or error) to return (or
+          next_send (outcome.Outcome): the value (or error) to return (or
             raise) from :func:`wait_task_rescheduled`.
 
         """
+        if next_send is self._NO_SEND:
+            next_send = Value(None)
+
         assert task._runner is self
         assert task._next_send is None
         task._next_send = next_send
@@ -713,9 +725,7 @@ class Runner:
         self.runq.append(task)
         self.instrument("task_scheduled", task)
 
-    def spawn_impl(
-        self, async_fn, args, nursery, name, *, ki_protection_enabled=False
-    ):
+    def spawn_impl(self, async_fn, args, nursery, name, *, system_task=False):
 
         ######
         # Make sure the nursery is in working order
@@ -737,7 +747,7 @@ class Runner:
         def _return_value_looks_like_wrong_library(value):
             # Returned by legacy @asyncio.coroutine functions, which includes
             # a surprising proportion of asyncio builtins.
-            if inspect.isgenerator(value):
+            if isinstance(value, collections.abc.Generator):
                 return True
             # The protocol for detecting an asyncio Future-like object
             if getattr(value, "_asyncio_future_blocking", None) is not None:
@@ -756,7 +766,7 @@ class Runner:
             coro = async_fn(*args)
         except TypeError:
             # Give good error for: nursery.start_soon(trio.sleep(1))
-            if inspect.iscoroutine(async_fn):
+            if isinstance(async_fn, collections.abc.Coroutine):
                 raise TypeError(
                     "trio was expecting an async function, but instead it got "
                     "a coroutine object {async_fn!r}\n"
@@ -789,7 +799,7 @@ class Runner:
         # for things like functools.partial objects wrapping an async
         # function. So we have to just call it and then check whether the
         # result is a coroutine object.
-        if not inspect.iscoroutine(coro):
+        if not isinstance(coro, collections.abc.Coroutine):
             # Give good error for: nursery.start_soon(func_returning_future)
             if _return_value_looks_like_wrong_library(coro):
                 raise TypeError(
@@ -798,6 +808,13 @@ class Runner:
                     "That won't work without some sort of compatibility shim."
                     .format(coro)
                 )
+
+            if isasyncgen(coro):
+                raise TypeError(
+                    "start_soon expected an async function but got an async "
+                    "generator {!r}".format(coro)
+                )
+
             # Give good error for: nursery.start_soon(some_sync_fn)
             raise TypeError(
                 "trio expected an async function, but {!r} appears to be "
@@ -818,28 +835,36 @@ class Runner:
                 name = "{}.{}".format(name.__module__, name.__qualname__)
             except AttributeError:
                 name = repr(name)
+
+        if system_task:
+            context = self.system_context.copy()
+        else:
+            context = copy_context()
+
         task = Task(
             coro=coro,
             parent_nursery=nursery,
             runner=self,
             name=name,
-            context=copy_context(),
+            context=context,
         )
         self.tasks.add(task)
+        coro.cr_frame.f_locals.setdefault(
+            LOCALS_KEY_KI_PROTECTION_ENABLED, system_task
+        )
+
         if nursery is not None:
             nursery._children.add(task)
             for scope in nursery._cancel_stack:
                 scope._add_task(task)
-        coro.cr_frame.f_locals.setdefault(
-            LOCALS_KEY_KI_PROTECTION_ENABLED, ki_protection_enabled
-        )
-        if nursery is not None:
+
             # Task locals are inherited from the spawning task, not the
             # nursery task. The 'if nursery' check is just used as a guard to
             # make sure we don't try to do this to the root task.
             parent_task = current_task()
             for local, values in parent_task._locals.items():
                 task._locals[local] = dict(values)
+
         self.instrument("task_spawned", task)
         # Special case: normally next_send should be a Result, but for the
         # very first send we have to send a literal unboxed None.
@@ -851,8 +876,14 @@ class Runner:
             task._cancel_stack[-1]._remove_task(task)
         self.tasks.remove(task)
         if task._parent_nursery is None:
-            # the init task should be the last task to exit
-            assert not self.tasks
+            # the init task should be the last task to exit. If not, then
+            # something is very wrong. Probably it hit some unexpected error,
+            # in which case we re-raise the error (which will later get
+            # converted to a TrioInternalError, but at least we'll get a
+            # traceback). Otherwise, raise a new error.
+            if self.tasks:  # pragma: no cover
+                result.unwrap()
+                raise TrioInternalError
         else:
             task._parent_nursery._child_finished(task, result)
         if task is self.main_task:
@@ -886,6 +917,8 @@ class Runner:
           *enabled*. If you want your task to be interruptible by control-C,
           then you need to use :func:`disable_ki_protection` explicitly.
 
+        * System tasks do not inherit context variables from their creator.
+
         Args:
           async_fn: An async callable.
           args: Positional arguments for ``async_fn``. If you want to pass
@@ -921,10 +954,11 @@ class Runner:
         if name is None:
             name = async_fn
         return self.spawn_impl(
-            system_task_wrapper, (async_fn, args),
+            system_task_wrapper,
+            (async_fn, args),
             self.system_nursery,
             name,
-            ki_protection_enabled=True
+            system_task=True,
         )
 
     async def init(self, async_fn, args):
@@ -1123,7 +1157,7 @@ def run(
     async_fn,
     *args,
     clock=None,
-    instruments=[],
+    instruments=(),
     restrict_keyboard_interrupt_to_checkpoints=False
 ):
     """Run a trio-flavored async function, and return the result.
@@ -1209,8 +1243,13 @@ def run(
         clock = SystemClock()
     instruments = list(instruments)
     io_manager = TheIOManager()
+    system_context = copy_context()
+    system_context.run(current_async_library_cvar.set, "trio")
     runner = Runner(
-        clock=clock, instruments=instruments, io_manager=io_manager
+        clock=clock,
+        instruments=instruments,
+        io_manager=io_manager,
+        system_context=system_context,
     )
     GLOBAL_RUN_CONTEXT.runner = runner
     locals()[LOCALS_KEY_KI_PROTECTION_ENABLED] = True
@@ -1255,10 +1294,11 @@ def run_impl(runner, async_fn, args):
     runner.instrument("before_run")
     runner.clock.start_clock()
     runner.init_task = runner.spawn_impl(
-        runner.init, (async_fn, args),
+        runner.init,
+        (async_fn, args),
         None,
         "<init>",
-        ki_protection_enabled=True
+        system_task=True,
     )
 
     # You know how people talk about "event loops"? This 'while' loop right
