@@ -6,8 +6,6 @@ from .abc import HalfCloseableStream
 __all__ = [
     "aclose_forcefully",
     "BrokenStreamError",
-    "ClosedStreamError",
-    "ClosedListenerError",
     "StapledStream",
 ]
 
@@ -25,7 +23,7 @@ async def aclose_forcefully(resource):
     cancelling it immediately.
 
     Most users won't need this, but it may be useful on cleanup paths where
-    you can't afford to block, or if you want to close an resource and don't
+    you can't afford to block, or if you want to close a resource and don't
     care about handling it gracefully. For example, if
     :class:`~trio.ssl.SSLStream` encounters an error and cannot perform its
     own graceful close, then there's no point in waiting to gracefully shut
@@ -43,14 +41,14 @@ async def aclose_forcefully(resource):
 
 
 class BrokenStreamError(Exception):
-    """Raised when an attempt to use a stream a stream fails due to external
+    """Raised when an attempt to use a stream fails due to external
     circumstances.
 
     For example, you might get this if you try to send data on a stream where
     the remote side has already closed the connection.
 
     You *don't* get this error if *you* closed the stream – in that case you
-    get :class:`ClosedStreamError`.
+    get :class:`ClosedResourceError`.
 
     This exception's ``__cause__`` attribute will often contain more
     information about the underlying error.
@@ -59,36 +57,6 @@ class BrokenStreamError(Exception):
     pass
 
 
-class ClosedStreamError(Exception):
-    """Raised when an attempt to use a stream fails because the stream was
-    already closed locally.
-
-    You *only* get this error if *your* code closed the stream object you're
-    attempting to use by calling
-    :meth:`~trio.abc.AsyncResource.aclose` or
-    similar. (:meth:`~trio.abc.SendStream.send_all` might also raise this if
-    you already called :meth:`~trio.abc.HalfCloseableStream.send_eof`.)
-    Therefore this exception generally indicates a bug in your code.
-
-    If a problem arises elsewhere, for example due to a network failure or a
-    misbehaving peer, then you get :class:`BrokenStreamError` instead.
-
-    """
-    pass
-
-
-class ClosedListenerError(Exception):
-    """Raised when an attempt to use a listener fails because it was already
-    closed locally.
-
-    You *only* get this error if *your* code closed the stream object you're
-    attempting to use by calling :meth:`~trio.abc.AsyncResource.aclose` or
-    similar. Therefore this exception generally indicates a bug in your code.
-
-    """
-    pass
-
-
 @attr.s(cmp=False, hash=False)
 class StapledStream(HalfCloseableStream):
     """This class `staples <https://en.wikipedia.org/wiki/Staple_(fastener)>`__
